/*  Canon Inkjet Printer Driver for Linux
 *  Copyright CANON INC. 2001-2012
 *  All Rights Reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 *
 * NOTE:
 *  - As a special exception, this program is permissible to link with the
 *    libraries released as the binary modules.
 *  - If you write modifications of your own for these programs, it is your
 *    choice whether to permit this exception to apply to your modifications.
 *    If you do not wish that, delete this exception.
 */


/*
 * Initial main.c file generated by Glade. Edit as required.
 * Glade will not overwrite this file.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef ENABLE_NLS
#  include <libintl.h>
#endif

#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <popt.h>

#include "callbacks.h"
#	include "interface.h"
#	include "support.h"


#define	DEFINE_GLOBALS
#include "bjuidefs.h"
#include "util_setconfig.h"

#define KEYTEXT_FILE "maintenance.res"

void SetGtkResourceFile();
void InitOption(int argc, char* argv[]);
void FreeOption();
void ShowModelDependWidgets();


int main(int argc, char *argv[])
{
	int ret = 1;

	g_main_window=NULL;	/* for UpdateMenuLink (check "first call") */
	//g_quality_dialog = NULL;	/* for UpdateMenuLink (check "first call") */

	bindtextdomain(PACKAGE, PACKAGE_LOCALE_DIR);
	bind_textdomain_codeset( PACKAGE, "UTF-8" );
	textdomain(PACKAGE);

	gtk_set_locale();

	// Set gtk resource file.
	SetGtkResourceFile();

	gtk_init(&argc, &argv);

	// Parse options.
	InitOption(argc, argv);

	// Set the directory name for storing keytext files.
	SetKeyTextDir(PACKAGE_DATA_DIR);

	// get Model ID
	g_model_id = ModelNameToID(PACKAGE_PRINTER_MODEL);

	// Load text resource.
	g_keytext_list = NULL;
	g_keytext_list = LoadKeyTextList(KEYTEXT_FILE);
	if ( g_keytext_list == NULL ) goto Err;
	

	// Get Printer Status
	SetMSIPaperSourceSettings();

	// Create main dialog, and realize it.
	g_main_window = create_ui_window();
	gtk_button_set_label( GTK_BUTTON(LookupWidget(g_main_window, "cancel_button")), 
			LookupText(g_keytext_list, "LUM_IDD_PHA_GUIDE_IDCANCEL"));
	gtk_widget_realize(g_main_window);

	// Set window title.
	snprintf(g_window_title, sizeof(g_window_title), "Canon %s", GetDispModelName());	//Ver.2.90(s)
	gtk_window_set_title( (GtkWindow *)g_main_window, g_window_title );

	// Show widgets depend on model.
	ShowUtilButtons(g_main_window);

	// Show main dialog
	gtk_widget_show(g_main_window);
	gtk_main();

	ret = 0;

Err:
	// Free key text dir
	FreeKeyTextDir();

	// Free option strings.
	FreeOption();

	return ret;
}

void SetGtkResourceFile()
{
	// Get gtk resources.
	const gchar* home_dir = NULL;
	gchar* rc_path = NULL;

	home_dir = g_get_home_dir();
	rc_path = g_strdup_printf("%s/.gtkrc", home_dir);
	gtk_rc_add_default_file(rc_path);
	g_free(rc_path);
}

#define	UI_OPT_SOCKET	1
#define	UI_OPT_INTVER	5
#define	UI_OPT_BIDI		6
#define	UI_OPT_DEVURI	7

void InitOption(int argc, char* argv[])
{
	static struct poptOption option_table[] =
	{
		{NULL,             's', POPT_ARG_STRING,  NULL,  UI_OPT_SOCKET },
//		{"model",           0,  POPT_ARG_STRING,  NULL,  UI_OPT_MODEL  },
//		{"cups",            0,  0,                NULL,  UI_OPT_CUPS   },
		{"internalversion", 0,  POPT_ARG_NONE,    NULL,  UI_OPT_INTVER },	/* Ver.2.70 */
		{"bidi",            0,  POPT_ARG_NONE,    NULL,  UI_OPT_BIDI   },	/* Ver.3.20 */
		{"devuri",          0,  POPT_ARG_STRING,  NULL,  UI_OPT_DEVURI },	/* Ver.3.70 */
		{NULL,              0,  0,                NULL,  0             }
	};

	poptContext context
		 = poptGetContext("maintenance", argc, (const char**)argv, option_table, 0);
	int rc;

	g_bidi_mode = FALSE;		/* Ver.3.20 */
	g_device_uri = NULL;		/* Ver.3.70 */

	while( (rc = poptGetNextOpt(context)) > 0 )
	{
		switch( rc )
		{
		case UI_OPT_SOCKET:
			g_socketname = g_strdup(poptGetOptArg(context));
			break;

		case UI_OPT_INTVER:
			g_message( "Internal Version : %s\n" ,INTERNAL_VERSION_STR );
			break;

		case UI_OPT_BIDI:		/* Ver.3.20 */
			g_bidi_mode = TRUE;
			break;

		case UI_OPT_DEVURI:
			g_device_uri = g_strdup(poptGetOptArg(context));
			break;

		default:
			break;
		}
	}
	
	if( rc < -1 )
		g_warning("Unknown switch, %s\n",
			 poptBadOption(context, POPT_BADOPTION_NOALIAS));
}

void FreeOption()
{
	if( g_device_uri != NULL ) {
		g_free(g_device_uri);
		g_device_uri = NULL;
	}
	if( g_socketname != NULL ) {
		g_free(g_socketname);
		g_socketname = NULL;
	}
}

